/*
# Copyright (C) 2015 audin.
#   2015/01: Modified for NUCLEO-F411RE demo.
#   http://mpu.seesaa.net
# Copyright (C) 2013 audin.
#   2013/07: Modified for LPC1114fn28 demo.
#   http://avr.paslog.jp
#
# This file is licensed under the MIT License.
*/

#include "sys.h"
#include "string.h"
#include "usart.h"
#include "Scheduler.h"

/*****************************************************************************
 * Private types/enumerations/variables
 ****************************************************************************/

static uint32_t ibaudrate;
static uint8_t rcvData;

volatile uint8_t fTxEnd, fRxEnd;
/*****************************************************************************
 * Public types/enumerations/variables
 ****************************************************************************/

/*****************************************************************************
 * Private functions
 ****************************************************************************/

/*****************************************************************************
 * Public functions
 ****************************************************************************/
void UART_putc( uint8_t c );

/* 
 * ===  FUNCTION  ======================================================================
 *         Name:  uart_init
 *  Description:  
 * =====================================================================================
 */

void uart_init( uint32_t baudrate)
{
	ibaudrate = baudrate;

	/* Setup UART for 8N1 */
    MX_USART2_UART_Init( baudrate );
    HAL_NVIC_SetPriority(USART2_IRQn, 0,1);
    HAL_NVIC_EnableIRQ(USART2_IRQn);
    HAL_UART_Receive_IT( &huart2, &rcvData, 1); 

#ifdef __XPRINTF__
	xfunc_out = UART_putc;  /* connect PutChar function to xprintf() */
#endif

}

/*******************

 * HAL_UART_TxCpltCallback
  
 *******************/
void HAL_UART_TxCpltCallback(UART_HandleTypeDef *huart)
{
    if (huart->Instance == USART2) {
        fTxEnd  = 1;
    }
}

/*******************

 * HAL_UART_RxCpltCallback
  
 *******************/
void HAL_UART_RxCpltCallback(UART_HandleTypeDef *huart)
{
    if (huart->Instance == USART2) {
        fRxEnd = 1;
    }
}

/*******************

 * USART2_IRQHandler
  
 *******************/
void USART2_IRQHandler(void) {
	HAL_UART_IRQHandler(&huart2);       
}
/* 
 * ===  FUNCTION  ======================================================================
 *         Name:  UART_putc
 *  Description:  
 * =====================================================================================
 */
void UART_putc( uint8_t c )
{
    fTxEnd = 0;
    HAL_UART_Transmit_IT( &huart2, &c, 1); 
    while( !fTxEnd ){ 
        yield();
    }
    wait(60); /* Delay for only demonstration purpose */
}

/* 
 * ===  FUNCTION  ======================================================================
 *         Name:  uartDrv_getchar
 *  Description:  
 * =====================================================================================
 */
uint8_t	uartDrv_getchar( void ){
    uint8_t c = 0;

    if( fRxEnd ){
        fRxEnd = 0;
        c = rcvData;
        HAL_UART_Receive_IT( &huart2, &rcvData, 1); 
    }
    return c;
}

/*******************

 * uartDrv_getBaudrate
  
 *******************/
uint32_t uartDrv_getBaudrate( void )
{
	return ibaudrate;
}


/* 
 * ===  FUNCTION  ======================================================================
 *         Name:  uartIsRxDataAvailable
 *  Description:  
 * =====================================================================================
 */
uint8_t uartIsRxDataAvailable(void){
 	
    return fRxEnd;
}

/**
 * @}
 */

