/*
#
# Copyright (C) 2010-2015, audin
# 2015/01: Modified for STM32
# 2013/07: Modified for LPCOpen Platform for LPC1114fn28
# 2011/04, 2012/11
# 2010/11
# http://avr.paslog.jp
# This file is licensed under the MIT License.
#
*/

#include "sys.h"
#include "uart.h"
#include "sio_tsk.h"
#include "platform.h"
#include "Scheduler.h"


#ifndef __AVR__
#	define printf_P	printf
#endif

#if defined( __XPRINTF__ )
#	include "xprintf.h"
#	undef printf
#	define printf xprintf
#elif defined( __SMALL_PRINTF__)
#	include "printf.h"
#else
#	include <stdio.h>
#endif



const char sVersion[] = "v0.2";
static int8_t state = 0;

const char PROGMEM msg_clr[]	= /*  */ "\x1b[2J";	//clear 
const char PROGMEM msg_os[]		= /*  */ "Scheduler %s, UART Transfer test. \n";
const char PROGMEM msg_cpu[]	= /*  */ "CPU may be STM32F4 or something.\n";
const char PROGMEM msg_wel[]	= /*  */ "Welcome !\n";
const char PROGMEM msg_bps[]	= /*  */ "UART speed is %u bps.\n"  ;
const char PROGMEM msg_led[]	= /*  */ "Now LED blink speed is %d.\n"  ;
const char PROGMEM msg_in[]		= /*  */ "\nPlease input LED blink speed [1 - 9]\n"  ;
const char PROGMEM msg_clk[]	= /*  */ "SystemClock = %d Hz\n"  ;



static int led_blink_speed = 9; 


void sio_key_input_tsk( void *pvParameters );

/*******************

* sio_Task
  
 *******************/
void sio_task( void ){
	sio_tsk( (void*)0 );	/* only stub */
}

extern int max_val;
void sio_tsk( void *pvParameters )
{
	( void ) pvParameters;	// not used

	while( 1 ){
		switch( state++ ){
		case -1:
			yield();							/* stop display */
			break;
		case 0:
			printf_P( msg_clr );				/* clear */
			wait( 2000 );						/* 2000[msec] */
			printf_P( msg_wel );
			wait( 2000 );
			break;
		case 1:
			printf_P( msg_os, sVersion );
			wait( 2000 );
			printf_P( msg_cpu );
			wait( 2000 );
			printf_P( msg_clk, (int)SystemCoreClock );
			wait( 2000 );
			break;
		case 2:
			printf_P( msg_bps, ( unsigned int )uartDrv_getBaudrate() );
			wait( 2000 );
			break;
		case 3:
			printf_P( msg_led, ( int )sio_get_led_blink_speed() );
			printf_P( msg_in );
			wait( 3000 );
			state = 0;
			break;
		}
	}
}

/*******************

* sio_Key_Input_tsk
  
 *******************/
void sio_key_input_task( void ){
	sio_key_input_tsk( (void*)0 );	/* Just stub */
}

void sio_key_input_tsk( void *pvParameters )
{
	char c;
	( void ) pvParameters;	// not used

	while(1){
		//vTaskSuspend( NULL );		
		c = uartDrv_getchar();
		if( (c  >= '1') && ('9' >= c ) ){
			led_blink_speed = c - '0';
		}
		yield();
	}
}

/*******************

* sio_get_led_blink_speed
  
 *******************/
int sio_get_led_blink_speed(void)
{
	return (int)led_blink_speed;
}

#if 0
/*******************

* sio_init
  
 *******************/
void sio_init( UART_DEV_t dev, uint32_t baudrate)
{
  uartDrv_init( dev,  baudrate );
}
#endif

